# Rectangle NN

We build a range-tree where the first two layers are on the `x-` and `y-` coordinates. Our data-structure on the third level of the tree is a Voronoi diagram with point-location.

1. Given a query `(R,q)`, we search the first two levels of our tree using `R`. This results in a set of sub-trees.
2.  We then query the root of each of these sub-trees using \(q\). Specifically, we use the Voronoi/point-location data-structure stored therein to find the point in the sub-tree with minimal distance to `q`.
3.  We return the point with minimum distance among all such sub-tree queries.

## Correctedness

After step (1), we have found a set of sub-trees. We return the point with minimum distance over all sub-trees. This corresponds to returning the point in the union of the sub-trees closest to \(q\). However, the first two-layers of our tree are simply a two-dimensional range-tree. Therefore the union of these sub-trees is exactly all the points \(p\in P\) residing in \(R\ \square\).

## Run-time

It takes \(O(\log^2 n)\) time to query the first two dimensions of our tree (vanilla 2D-range tree query). This results in \(O(\log^2 n)\) sub-trees roots. We perform a point-location query on the Voronoi-diagram of each sub-tree root. This costs us \(O(\log i )\) where `i` is the size of the sub-tree. \(O(\log i) \in O(\log n)\), therefore we perform an \(O(\log n)\) operation \(O(\log^2 n)\) times for a total of \(O(\log^3 n) \square\). This is expected if we use a trapezoidal-decomp or worst-case with a slab-decomp.

This is to be expected. A regular 3D range tree takes \(O(\log^3 n)\) (with no fractional-cascading), and a search for a 1D tree is as computationally expensive as a point-location query. 

## Preprocessing

Consider building a single y-tree. We first take \(O(n\log n)\) factor to go from points to the y-tree, but this is swallowed up by the Voronoi step. For input \(O(n)\), forming the Voronoi diagram and build a point-decomp takes \(O(n\log n)\) for trapezoidal-decomp and \(O(n^2\log n)\) for the slab method (we grabbed the construction time of a slab-decomp from the original paper).

If we just try and build every node Voronoi data-structure independently, we get the following:

**Trapezoidal**: Our recursive run-time looks like \(R(n) = 2R(n/2) + O(n\log n)\) expected. This solves to give us a pre-processing time of \(O(n\log^2 n)\).

**Slab**: Our recursion is \(R(n) = 2R(n/2) + O(n^2\log n)\). This solves to be \(O(n^2\log n)\).

If we then consider building the whole tree, the runtime is \(R(n) = 2R(n/2) + Y_\text{time}(n)\).

**Trapezoidal**: \(R(n) = 2R(n/2) + O(n\log^2 n)\) which solves to be \(O(n\log^3 n)\).

**Slab**: \(R(n) = 2R(n/2) + O(n^2\log n)\) which solves to be \(O(n^2\log n)\).